IMAGES_FOLDER = 'visual_images'
ACTUAL_IMAGE_BASE_FOLDER = 'actual'
DIFF_IMAGE_BASE_FOLDER = 'diff'
BASELINE_IMAGE_BASE_FOLDER = 'baseline'
MODE_TEST = 'test'
MODE_BASELINE = 'baseline'
REPORT_FILE = 'visualReport.html'
REPORT_EXPIRATION_THRESHOLD = 10
IMAGE_EXTENSIONS = ['png', 'jpg', 'jpeg', 'bmp', 'raw', 'pdf']
ROBOT_LIBRARY_SCOPE = 'GLOBAL'
ROBOT_LISTENER_API_VERSION = 3